import pycurl

class Handle:
    def __init__(self, handle: pycurl.Curl) -> None: ...
    def setopt(self, option: int, value: object, /) -> None: ...
    def unsetopt(self, option: int, /) -> None: ...
    def getinfo(self, option: int, /) -> object: ...
    def getinfo_raw(self, option: int, /) -> object: ...
    def pause(self, state: int, /) -> None: ...



from ._enums import MILLISECONDS as MILLISECONDS, SECONDS as SECONDS, SocketEvt as SocketEvt, Time as Time
from ._exceptions import CurlError as CurlError
from .abc import RequestProtocol as RequestProtocol
from anyio.abc import ObjectReceiveStream as ObjectReceiveStream, ObjectSendStream as ObjectSendStream
from kodo.quantities import Quantity as Quantity
from socket import socket as Socket
from typing import Final, Self, TypeAlias, TypeVar

U = TypeVar('U')
R = TypeVar('R')
Event: TypeAlias
INFO_READ_SIZE: Final[int]

class Multi:
    def __init__(self) -> None: ...
    async def process(self, request: RequestProtocol[U, R]) -> U | R: ...

class _ExternalSocket(Socket):
    def __del__(self) -> None: ...
    @classmethod
    def from_fd(cls, fd: int, /) -> Self: ...



from typing import Protocol, TypeVar

U_co = TypeVar('U_co', covariant=True)
R_co = TypeVar('R_co', covariant=True)

class ConfigHandle(Protocol):
    def setopt(self, option: int, value: object, /) -> None: ...
    def unsetopt(self, option: int, /) -> None: ...
    def pause(self, state: int, /) -> None: ...

class GetInfoHandle(Protocol):
    def getinfo(self, option: int, /) -> object: ...
    def getinfo_raw(self, option: int, /) -> object: ...

class RequestProtocol(Protocol[U_co, R_co]):
    def configure_handle(self, handle: ConfigHandle, /) -> None: ...
    def has_update(self, /) -> bool: ...
    def get_update(self, /) -> U_co: ...
    def completed(self, handle: GetInfoHandle, /) -> R_co: ...



class CurlError(Exception):
    args: tuple[int, str]
    def __init__(self, code: int, msg: str) -> None: ...
    @property
    def code(self) -> int: ...
    @property
    def msg(self) -> str: ...



from .abc import ConfigHandle as ConfigHandle, GetInfoHandle as GetInfoHandle
from typing import IO

class Request:
    url: str
    destination: IO[bytes]
    def __init__(self, url: str, destination: IO[bytes]) -> None: ...
    def configure_handle(self, handle: ConfigHandle, /) -> None: ...
    def has_update(self, /) -> bool: ...
    def get_update(self, /) -> None: ...
    def completed(self, handle: GetInfoHandle, /) -> None: ...
